@@ -1,7 +1,6 @@
 /*
- * (C) Copyright 2008-2013
- * Stefano Babic, DENX Software Engineering, sbabic@denx.de.
- * 	on behalf of ifm electronic GmbH
+ * (C) Copyright 2013-2023
+ * Stefano Babic <stefano.babic@swupdate.org>
  *
  * SPDX-License-Identifier:     GPL-2.0-only
  */
@@ -68,9 +67,9 @@ static pthread_t network_thread_id;
  * reception of an install request
  *
  */
-
+bool stream_wkup = false;
 pthread_mutex_t stream_mutex = PTHREAD_MUTEX_INITIALIZER;
-pthread_cond_t stream_wkup = PTHREAD_COND_INITIALIZER;
+pthread_cond_t stream_cond = PTHREAD_COND_INITIALIZER;
 
 static struct installer inst;
 
@@ -184,7 +183,7 @@ static int extract_files(int fd, struct swupdate_cfg *software)
 				return -1;
 			}
 
-			if (check_hw_compatibility(software)) {
+			if (check_hw_compatibility(&software->hw, &software->hardware)) {
 				ERROR("SW not compatible with hardware");
 				return -1;
 			}
@@ -205,7 +204,7 @@ static int extract_files(int fd, struct swupdate_cfg *software)
 				 * to 512 bytes from the socket until the
 				 * client stops writing
 			 	 */
-				extract_padding(fd, &offset);
+				extract_padding(fd);
 				status = STREAM_END;
 				break;
 			}
@@ -535,7 +534,10 @@ void *network_initializer(void *data)
 
 		/* wait for someone to issue an install request */
 		pthread_mutex_lock(&stream_mutex);
-		pthread_cond_wait(&stream_wkup, &stream_mutex);
+		while (stream_wkup != true) {
+			pthread_cond_wait(&stream_cond, &stream_mutex);
+		}
+		stream_wkup = false;
 		inst.status = RUN;
 		pthread_mutex_unlock(&stream_mutex);
 		notify(START, RECOVERY_NO_ERROR, INFOLEVEL, "Software Update started !");
@@ -658,6 +660,15 @@ void *network_initializer(void *data)
 			update_transaction_state(software, STATE_IN_PROGRESS);
 
 			notify(RUN, RECOVERY_NO_ERROR, INFOLEVEL, "Installation in progress");
+
+			/*
+			 * if this update is signalled to be without reboot (On the Fly),
+			 * send a notification via progress for processes responsible for booting
+			 */
+			if (!software->reboot_required) {
+				swupdate_progress_info(RUN, CAUSE_REBOOT_MODE , "{ \"reboot-mode\" : \"no-reboot\"}");
+			}
+
 			ret = install_images(software);
 			if (ret != 0) {
 				update_transaction_state(software, STATE_FAILED);
@@ -702,6 +713,38 @@ void *network_initializer(void *data)
 		pthread_mutex_unlock(&stream_mutex);
 		TRACE("Main thread sleep again !");
 		notify(IDLE, RECOVERY_NO_ERROR, INFOLEVEL, "Waiting for requests...");
+
+		/*
+		 * Last step, if no restart is required,
+		 * SWUpdate can send automatically the feedback.
+		 * They are running on separate processes and should first
+		 * know that the update is completed.
+		 * It seems safe enough to wait uncoditionally for some
+		 * time, enough to let all SWUpdate's processes to be scheduled,
+		 * before sending the IPC message
+		 */
+		if (req->source == SOURCE_SURICATTA &&
+		    /*simple check without JSON */ strstr(req->info, "hawkbit") &&
+		    !software->reboot_required) {
+			ipc_message msg;
+			size_t size = sizeof(msg.data.procmsg.buf);
+			char *buf = msg.data.procmsg.buf;
+			memset(&msg, 0, sizeof(msg));
+			msg.magic = IPC_MAGIC;
+			msg.data.procmsg.source = SOURCE_SURICATTA;
+			msg.data.procmsg.cmd = CMD_ACTIVATION;
+			msg.type = SWUPDATE_SUBPROCESS;
+			snprintf(buf, size, "{ \"status\" : \"%s\", \"finished\" : \"%s\" ,\"execution\" : \"%s\" ,\"details\" : [ ]}",
+				"1",
+				inst.last_install == SUCCESS ? "success" : "failure",
+				"closed"
+				 );
+			sleep(2);
+			TRACE("SEND CONCLUSION TO HAWKBIT");
+			ipc_send_cmd(&msg);
+		}
+
+
 	}
 
 	pthread_exit((void *)0);
