@@ -39,6 +39,7 @@
 #include "mongoose_interface.h"
 #include "download_interface.h"
 #include "network_ipc.h"
+#include "network_utils.h"
 #include "sslapi.h"
 #include "suricatta/suricatta.h"
 #include "delta_process.h"
@@ -50,6 +51,7 @@
 #include "bootloader.h"
 #include "versions.h"
 #include "hw-compatibility.h"
+#include "swupdate_vars.h"
 
 #ifdef CONFIG_SYSTEMD
 #include <systemd/sd-daemon.h>
@@ -150,6 +152,7 @@ static void usage(char *programname)
 		" -l, --loglevel <level>         : logging level\n"
 		" -L, --syslog                   : enable syslog logger\n"
 #ifdef CONFIG_SIGNED_IMAGES
+#ifndef CONFIG_SIGALG_GPG
 		" -k, --key <public key file>    : file with public key to verify images\n"
 		"     --cert-purpose <purpose>   : set expected certificate purpose\n"
 		"                                  [emailProtection|codeSigning] (default: emailProtection)\n"
@@ -158,6 +161,7 @@ static void usage(char *programname)
 #endif
 		"     --ca-path                  : path to the Certificate Authority (PEM)\n"
 #endif
+#endif
 #ifdef CONFIG_ENCRYPTED_IMAGES
 		" -K, --key-aes <key file>       : the file contains the symmetric key to be used\n"
 		"                                  to decrypt images\n"
@@ -314,11 +318,25 @@ static int read_globals_settings(void *elem, void *data)
 				"postupdatecmd", sw->postupdatecmd);
 	GET_FIELD_STRING(LIBCFG_PARSER, elem,
 				"preupdatecmd", sw->preupdatecmd);
+	GET_FIELD_STRING(LIBCFG_PARSER, elem,
+				"namespace-vars", sw->namespace_for_vars);
+	if (strlen(sw->namespace_for_vars)) {
+		if (!swupdate_set_default_namespace(sw->namespace_for_vars))
+			WARN("Default Namaspace for SWUpdate vars cannot be set, possible side-effects");
+	}
+
 	get_field(LIBCFG_PARSER, elem, "verbose", &sw->verbose);
 	get_field(LIBCFG_PARSER, elem, "loglevel", &sw->loglevel);
 	get_field(LIBCFG_PARSER, elem, "syslog", &sw->syslog_enabled);
 	GET_FIELD_STRING(LIBCFG_PARSER, elem,
 				"no-downgrading", sw->minimum_version);
+	tmp[0] = '\0';
+	GET_FIELD_STRING(LIBCFG_PARSER, elem,
+				"fwenv-config-location", tmp);
+	if (strlen(tmp)) {
+		set_fwenv_config(tmp);
+		tmp[0] = '\0';
+	}
 	if (strlen(sw->minimum_version))
 		sw->no_downgrading = true;
 	GET_FIELD_STRING(LIBCFG_PARSER, elem,
@@ -338,6 +356,10 @@ static int read_globals_settings(void *elem, void *data)
 
 	char software_select[SWUPDATE_GENERAL_STRING_SIZE] = "";
 	GET_FIELD_STRING(LIBCFG_PARSER, elem, "select", software_select);
+	GET_FIELD_STRING(LIBCFG_PARSER, elem,
+				"gpg-home-dir", sw->gpg_home_directory);
+	GET_FIELD_STRING(LIBCFG_PARSER, elem,
+				"gpgme-protocol", sw->gpgme_protocol);
 	if (software_select[0] != '\0') {
 		/* by convention, errors in a configuration section are ignored */
 		(void)parse_image_selector(software_select, sw);
@@ -470,9 +492,11 @@ int main(int argc, char **argv)
 	strcat(main_options, "H:");
 #endif
 #ifdef CONFIG_SIGNED_IMAGES
+#ifndef CONFIG_SIGALG_GPG
 	strcat(main_options, "k:");
 	public_key_mandatory = 1;
 #endif
+#endif
 #ifdef CONFIG_ENCRYPTED_IMAGES
 	strcat(main_options, "K:");
 #endif
@@ -746,6 +770,19 @@ int main(int argc, char **argv)
 		exit(EXIT_FAILURE);
 	}
 
+#ifdef CONFIG_SIGALG_GPG
+	if (!strlen(swcfg.gpg_home_directory)) {
+		fprintf(stderr,
+			 "Error: SWUpdate is built for signed images, provide a GnuPG home directory.\n");
+		exit(EXIT_FAILURE);
+	}
+	if (!strlen(swcfg.gpgme_protocol)) {
+		fprintf(stderr,
+			"Error: SWUpdate is built for signed images, please specify GnuPG protocol.\n");
+		exit(EXIT_FAILURE);
+	}
+#endif
+
 	if (opt_c && !opt_i) {
 		fprintf(stderr,
 			"Error: Checking local images requires -i <file>.\n");
@@ -767,7 +804,7 @@ int main(int argc, char **argv)
 
 	swupdate_crypto_init();
 
-	if (strlen(swcfg.publickeyfname)) {
+	if (strlen(swcfg.publickeyfname) || strlen(swcfg.gpg_home_directory)) {
 		if (swupdate_dgst_init(&swcfg, swcfg.publickeyfname)) {
 			fprintf(stderr,
 				 "Error: Crypto cannot be initialized.\n");
@@ -849,6 +886,9 @@ int main(int argc, char **argv)
 	 *  Start daemon if just a check is required
 	 *  SWUpdate will exit after the check
 	 */
+	if(init_socket_unlink_handler() != 0){
+		TRACE("Cannot setup socket cleanup on exit, sockets won't be unlinked.");
+	}
 	network_daemon = start_thread(network_initializer, &swcfg);
 
 	start_thread(progress_bar_thread, NULL);
@@ -889,7 +929,7 @@ int main(int argc, char **argv)
 		read_settings_user_id(&handle, "download", &uid, &gid);
 		start_subprocess(SOURCE_DOWNLOADER, "download", uid, gid,
 				 cfgfname, dwlac,
-				 dwlav, start_download);
+				 dwlav, start_download_server);
 		freeargs(dwlav);
 	}
 #endif
@@ -930,9 +970,7 @@ int main(int argc, char **argv)
 	}
 
 #ifdef CONFIG_SYSTEMD
-	if (sd_booted()) {
-		sd_notify(0, "READY=1");
-	}
+	sd_notify(0, "READY=1");
 #endif
 
 	/*
