@@ -53,6 +53,12 @@ struct decryption_key {
 
 static struct decryption_key *aes_key = NULL;
 
+/*
+ * Configuration file for fw_env.config
+ */
+
+static char *fwenv_config = NULL;
+
 char *sdup(const char *str) {
 	char *p;
 	if ((p = (char *) malloc(strlen(str) + 1)) != NULL) {
@@ -156,8 +162,8 @@ void swupdate_create_directory(const char* path) {
 		return;
 	}
 	if (mkdir(dpath, 0777)) {
-		WARN("Directory %s cannot be created due to : %s",
-			 path, strerror(errno));
+		WARN("Directory %s cannot be created: %s",
+			 dpath, strerror(errno));
 	}
 	free(dpath);
 }
@@ -557,22 +563,27 @@ int set_aes_key(const char *key, const char *ivt)
 	return 0;
 }
 
-int set_aes_ivt(const char *ivt)
-{
-	int ret;
-
-	if (!aes_key)
-		return -EFAULT;
-
-	ret = ascii_to_bin(aes_key->ivt, sizeof(aes_key->ivt), ivt);
-
-	if (ret) {
-		return -EINVAL;
-	}
-
-	return 0;
+const char *get_fwenv_config(void) {
+	if (!fwenv_config)
+#if defined(CONFIG_UBOOT)
+		return CONFIG_UBOOT_FWENV;
+#else
+		return NULL;
+#endif
+	return fwenv_config;
 }
 
+void set_fwenv_config(const char *fname) {
+	if (!fname)
+		return;
+
+	if (fwenv_config)
+		free(fwenv_config);
+
+	fwenv_config = strdup(fname);
+}
+
+
 char** string_split(const char* in, const char d)
 {
 	char** result = 0;
@@ -749,6 +760,9 @@ int swupdate_mount(const char *device, const char *dir, const char *fstype)
 	return nmount(iov, iovlen, mntflags);
 #else
 	/* Not implemented for this OS, no specific errno. */
+	(void)device;
+	(void)dir;
+	(void)fstype;
 	errno = 0;
 	return -1;
 #endif
@@ -763,6 +777,7 @@ int swupdate_umount(const char *dir)
 	return unmount(dir, mntflags);
 #else
 	/* Not implemented for this OS, no specific errno. */
+	(void)dir;
 	errno = 0;
 	return -1;
 #endif
@@ -1211,3 +1226,10 @@ bool img_check_free_space(struct img_type *img, int fd)
 
 	return check_free_space(fd, size, img->fname);
 }
+
+bool check_same_file(int fd1, int fd2) {
+    struct stat stat1, stat2;
+    if(fstat(fd1, &stat1) < 0) return false;
+    if(fstat(fd2, &stat2) < 0) return false;
+    return (stat1.st_dev == stat2.st_dev) && (stat1.st_ino == stat2.st_ino);
+}
