@@ -1,7 +1,6 @@
 /*
- * (C) Copyright 2013
- * Stefano Babic, DENX Software Engineering, sbabic@denx.de.
- * 	on behalf of ifm electronic GmbH
+ * (C) Copyright 2013-2023
+ * Stefano Babic <stefano.babic@swupdate.org>
  *
  * SPDX-License-Identifier:     GPL-2.0-only
  */
@@ -24,7 +23,7 @@
 #include "bootloader.h"
 #include "progress.h"
 #include "swupdate_image.h"
-#include "hw-compatibility.h"
+#include "swupdate.h"
 
 #define LUA_TYPE_PEMBSCR 1
 #define LUA_TYPE_HANDLER 2
@@ -64,16 +63,13 @@ struct stat_fields {
 	stat_push_function push;
 };
 
-#ifdef CONFIG_HANDLER_IN_LUA
 static int l_register_handler( lua_State *L );
 static int l_call_handler(lua_State *L);
-#endif
 static void image2table(lua_State* L, struct img_type *img);
 static void table2image(lua_State* L, struct img_type *img);
 static void update_table(lua_State* L, struct img_type *img);
 static int luaopen_swupdate(lua_State *L);
 
-#ifdef CONFIG_HANDLER_IN_LUA
 static bool is_type(lua_State *L, uintptr_t type)
 {
 	lua_getglobal(L, "SWUPDATE_LUA_TYPE");
@@ -81,7 +77,6 @@ static bool is_type(lua_State *L, uintptr_t type)
 	lua_pop(L, 1);
 	return ret;
 }
-#endif
 
 static void lua_dump_table(lua_State *L, char *str, struct img_type *img, const char *key)
 {
@@ -191,7 +186,6 @@ void LUAstackDump(lua_State *L)
 	}
 }
 
-#ifdef CONFIG_HANDLER_IN_LUA
 static void lua_report_exception(lua_State *L)
 {
 	const char *s = lua_tostring(L, -1);
@@ -206,7 +200,6 @@ static void lua_report_exception(lua_State *L)
 		s += len;
 	} while (*s++);
 }
-#endif
 
 int run_lua_script(const char *script, const char *function, char *parms)
 {
@@ -374,7 +367,6 @@ static void lua_number_to_img(struct img_type *img, const char *key,
 		img->skip = (unsigned int)val;
 }
 
-#ifdef CONFIG_HANDLER_IN_LUA
 static int l_copy2file(lua_State *L)
 {
 	luaL_checktype(L, 1, LUA_TTABLE);
@@ -393,6 +385,12 @@ static int l_copy2file(lua_State *L)
 	uint32_t checksum = 0;
 
 	table2image(L, &img);
+	if (check_same_file(img.fdin, fdout)) {
+		lua_pop(L, 1);
+		lua_pushinteger(L, -1);
+		lua_pushstring(L, "Output file path is same as input file temporary path");
+		goto copyfile_exit;
+	}
 	int ret = copyfile(img.fdin,
 				 &fdout,
 				 img.size,
@@ -497,7 +495,6 @@ static int l_istream_read(lua_State* L)
 	lua_pushnil(L);
 	return 2;
 }
-#endif
 
 static void update_table(lua_State* L, struct img_type *img)
 {
@@ -565,7 +562,6 @@ static void update_table(lua_State* L, struct img_type *img)
 		}
 		lua_settable(L, -3);
 
-#ifdef CONFIG_HANDLER_IN_LUA
 		if (is_type(L, LUA_TYPE_HANDLER)) {
 			lua_pushstring(L, "copy2file");
 			lua_pushcfunction(L, &l_copy2file);
@@ -575,7 +571,6 @@ static void update_table(lua_State* L, struct img_type *img)
 			lua_pushcfunction(L, &l_istream_read);
 			lua_settable(L, -3);
 		}
-#endif
 
 		lua_getfield(L, -1, "_private");
         LUA_PUSH_IMG_NUMBER(img, "offset", offset);
@@ -589,7 +584,6 @@ static void update_table(lua_State* L, struct img_type *img)
 	}
 }
 
-#ifdef CONFIG_HANDLER_IN_LUA
 #if LUA_VERSION_NUM > 501
 static int l_istream_fclose(lua_State *L)
 {
@@ -598,7 +592,6 @@ static int l_istream_fclose(lua_State *L)
 	return 1;
 }
 #endif
-#endif
 
 static void image2table(lua_State* L, struct img_type *img)
 {
@@ -626,7 +619,6 @@ static void image2table(lua_State* L, struct img_type *img)
 
 		update_table(L, img);
 
-#ifdef CONFIG_HANDLER_IN_LUA
 		if (is_type(L, LUA_TYPE_HANDLER)) {
 			lua_getfield(L, -1, "_private");
 			lua_pushstring(L, "istream");
@@ -643,7 +635,6 @@ static void image2table(lua_State* L, struct img_type *img)
 			lua_settable(L, -3);
 			lua_pop(L, 1);
 		}
-#endif
 	}
 }
 
@@ -677,7 +668,6 @@ static void table2image(lua_State* L, struct img_type *img) {
 		lua_getfield(L, -1, "_private");
 		lua_getfield(L, -1, "offset");
 		img->offset = (off_t)luaL_checknumber(L, -1);
-#ifdef CONFIG_HANDLER_IN_LUA
 		if (is_type(L, LUA_TYPE_HANDLER)) {
 			lua_pop(L, 1);
 			lua_getfield(L, -1, "istream");
@@ -688,7 +678,6 @@ static void table2image(lua_State* L, struct img_type *img) {
 				img->fdin = fileno(lstream->f);
 			}
 		}
-#endif
 		lua_pop(L,2);
 	}
 }
@@ -935,7 +924,6 @@ static int l_get_selection(lua_State *L) {
 }
 
 
-#ifdef CONFIG_HANDLER_IN_LUA
 static int l_get_tmpdir(lua_State *L)
 {
 	lua_pushstring(L, get_tmpdir());
@@ -954,7 +942,6 @@ static int l_progress_update(lua_State *L)
 	swupdate_progress_update((unsigned int) percent);
 	return 0;
 }
-#endif
 
 static int l_get_hw(lua_State *L)
 {
@@ -1208,7 +1195,6 @@ static const luaL_Reg l_swupdate_bootenv[] = {
         { NULL, NULL }
 };
 
-#ifdef CONFIG_HANDLER_IN_LUA
 static const luaL_Reg l_swupdate_handler[] = {
         { "register_handler", l_register_handler },
         { "call_handler", l_call_handler },
@@ -1217,7 +1203,6 @@ static const luaL_Reg l_swupdate_handler[] = {
 	{ "progress_update", l_progress_update },
         { NULL, NULL }
 };
-#endif
 
 /**
  * @brief function to register the swupdate package in the Lua Stack
@@ -1252,7 +1237,6 @@ static int luaopen_swupdate(lua_State *L)
 	lua_push_enum(L, "PARTLABEL", ROOT_DEV_PARTLABEL);
 	lua_settable(L, -3);
 
-#ifdef CONFIG_HANDLER_IN_LUA
 	if (is_type(L, LUA_TYPE_HANDLER)) {
 		/* register handler-specific functions to swupdate module table. */
 		luaL_setfuncs(L, l_swupdate_handler, 0);
@@ -1278,12 +1262,10 @@ static int luaopen_swupdate(lua_State *L)
 		}
 		lua_settable(L, -3);
 	}
-#endif
 
 	return 1;
 }
 
-#ifdef CONFIG_HANDLER_IN_LUA
 static lua_State *gL = NULL;
 
 /**
@@ -1511,9 +1493,6 @@ int lua_handlers_init(void)
 
 	return ret;
 }
-#else
-int lua_handlers_init(void) {return 0;}
-#endif
 
 lua_State *lua_parser_init(const char *buf, struct dict *bootenv)
 {
