@@ -15,9 +15,9 @@
 #include <string.h>
 #include <zmq.h>
 
-#include <swupdate.h>
-#include <handler.h>
-#include <util.h>
+#include "handler.h"
+#include "util.h"
+#include "swupdate_image.h"
 
 #define MSG_FRAMES	2
 #define FRAME_CMD	0
@@ -129,7 +129,7 @@ static int RHmsg_get_ack(struct RHmsg *self, void *request)
 	return 0;
 }
 
-static int forward_data(void *request, const void *buf, unsigned int len)
+static int forward_data(void *request, const void *buf, size_t len)
 {
 	struct RHmsg RHmessage;
 	int ret;
