@@ -1,12 +1,11 @@
 /*
- * (C) Copyright 2016
- * Stefano Babic, DENX Software Engineering, sbabic@denx.de.
+ * (C) Copyright 2016-2023
+ * Stefano Babic, <stefano.babic@swupdate.org>
  *
  * SPDX-License-Identifier:     GPL-2.0-only
  */
 
-#ifndef _INSTALL_PROGRESS_H
-#define _INSTALL_PROGRESS_H
+#pragma once
 
 #include <swupdate_status.h>
 #include <progress_ipc.h>
@@ -28,5 +27,3 @@ void swupdate_progress_info(RECOVERY_STATUS status, int cause, const char *msg);
 void swupdate_download_update(unsigned int perc, unsigned long long totalbytes, sourcetype source);
 
 void *progress_bar_thread (void *data);
-
-#endif
