@@ -1,21 +1,23 @@
 /*
- * (C) Copyright 2012-2016
- * Stefano Babic, DENX Software Engineering, sbabic@denx.de.
+ * (C) Copyright 2012-2023
+ * Stefano Babic <stefano.babic@swupdate.org>
  *
  * SPDX-License-Identifier:     GPL-2.0-only
  */
 
-#ifndef _UTIL_H
-#define _UTIL_H
+#pragma once
 
+#include <fcntl.h>
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
 #include <stdbool.h>
+#include <sys/time.h>
 #if defined(__linux__)
 #include <linux/types.h>
 #endif
+#include <sys/types.h>
 #include "globals.h"
 #include "swupdate_status.h"
 #include "swupdate_dict.h"
@@ -151,6 +153,7 @@ void notifier_set_color(int level, char *col);
 
 
 #define IS_STR_EQUAL(s,s1) (s && s1 && !strcmp(s,s1))
+#define UNUSED __attribute__((__unused__))
 
 #define LG_16 4
 #define FROM_HEX(f) from_ascii (f, sizeof f, LG_16)
@@ -229,6 +232,11 @@ int read_lines_notify(int fd, char *buf, int buf_size, int *buf_offset,
 		      LOGLEVEL level);
 long long get_output_size(struct img_type *img, bool strict);
 bool img_check_free_space(struct img_type *img, int fd);
+bool check_same_file(int fd1, int fd2);
+
+/* location for libubootenv configuration file */
+const char *get_fwenv_config(void);
+void set_fwenv_config(const char *fname);
 
 /* Decryption key functions */
 int load_decryption_key(char *fname);
@@ -236,7 +244,6 @@ unsigned char *get_aes_key(void);
 char get_aes_keylen(void);
 unsigned char *get_aes_ivt(void);
 int set_aes_key(const char *key, const char *ivt);
-int set_aes_ivt(const char *ivt);
 
 /* Getting global information */
 int get_install_info(sourcetype *source, char *buf, size_t len);
@@ -265,4 +272,3 @@ int swupdate_umount(const char *dir);
 
 /* Date / Time utilities */
 char *swupdate_time_iso8601(struct timeval *tv);
-#endif
