@@ -31,8 +31,6 @@
 #define NODEROOT (!strlen(CONFIG_PARSERROOT) ? \
 			"software" : CONFIG_PARSERROOT)
 
-#if defined(CONFIG_LIBCONFIG) || defined(CONFIG_JSON)
-
 typedef int (*parse_element)(parsertype p,
 		void *cfg, void *start, const char **nodes,
 		struct swupdate_cfg *swcfg, lua_State *L);
@@ -214,6 +212,19 @@ static bool get_common_fields(parsertype p, void *cfg, struct swupdate_cfg *swcf
 		}
 	}
 
+	/*
+	 * As default, reboot is initiated
+	 */
+	swcfg->reboot_required = true;
+	if((setting = find_node(p, cfg, "reboot", swcfg)) != NULL) {
+		get_field(p, setting, NULL, &swcfg->reboot_required);
+	}
+
+	TRACE("rebot_required %d", swcfg->reboot_required);
+
+	/*
+	 * Check if SWU should be cached
+	 */
 	if ((setting = find_node(p, cfg, "output", swcfg)) != NULL) {
 		if (!strlen(swcfg->output)) {
 			TRACE("Output file set but not enabled with -o, ignored");
@@ -224,6 +235,11 @@ static bool get_common_fields(parsertype p, void *cfg, struct swupdate_cfg *swcf
 		}
 	}
 
+	if((setting = find_node(p, cfg, "namespace-for-vars", swcfg)) != NULL) {
+		GET_FIELD_STRING(p, setting, NULL, swcfg->namespace_for_vars);
+		TRACE("Namespaced used to store SWUpdate's vars: %s", swcfg->namespace_for_vars);
+	}
+
 	return true;
 }
 
@@ -398,8 +414,6 @@ static int parse_common_attributes(parsertype p, void *elem, struct img_type *im
 	GET_FIELD_STRING(p, elem, "mtdname", image->mtdname);
 	GET_FIELD_STRING(p, elem, "filesystem", image->filesystem);
 	GET_FIELD_STRING(p, elem, "type", image->type);
-	get_field(p, elem, "offset", &offset);
-	GET_FIELD_STRING(p, elem, "offset", seek_str);
 	GET_FIELD_STRING(p, elem, "data", image->type_data);
 	get_hash_value(p, elem, image->sha256);
 
@@ -407,9 +421,11 @@ static int parse_common_attributes(parsertype p, void *elem, struct img_type *im
 	 * offset can be set as number or string. As string,
 	 * multiplier suffixes are allowed
 	 */
-	if (offset)
+	if (is_field_numeric(p, elem, "offset")) {
+		get_field(p, elem, "offset", &offset);
 		image->seek = offset;
-	else {
+	} else {
+		GET_FIELD_STRING(p, elem, "offset", seek_str);
 		/* convert the offset handling multiplicative suffixes */
 		image->seek = ustrtoull(seek_str, NULL, 0);
 		if (errno){
@@ -719,6 +735,82 @@ static int parse_bootloader(parsertype p, void *cfg, struct swupdate_cfg *swcfg,
 	return _parse_bootloader(p, cfg, setting, nodes, swcfg, L);
 }
 
+static int _parse_vars(parsertype p, void *cfg, void *setting, const char **nodes, struct swupdate_cfg *swcfg, lua_State *L)
+{
+	void *elem;
+	int count, i, skip, err;
+	struct img_type dummy;
+
+	if (setting == NULL) {
+		return 0;
+	}
+
+	count = get_array_length(p, setting);
+
+	for(i = (count - 1); i >= 0; --i) {
+		elem = get_elem_from_idx(p, setting, i);
+
+		if (!elem)
+			continue;
+
+		if (exist_field_string(p, elem, "ref")) {
+			err = parser_follow_link(p, cfg, elem, nodes, swcfg, _parse_bootloader, L);
+			if (err)
+				return err;
+			continue;
+		}
+
+		/*
+		 * dummy is just used for hooks
+		 */
+		memset(&dummy, 0, sizeof(dummy));
+
+		/*
+		 * Check for mandatory field
+		 */
+		if (!exist_field_string(p, elem, "name")) {
+		    ERROR("vars must have name field");
+		    return -1;
+		}
+
+		/*
+		 * Call directly get_field_string with size 0
+		 * to let allocate the place for the strings
+		 */
+		GET_FIELD_STRING(p, elem, "name", dummy.id.name);
+		GET_FIELD_STRING(p, elem, "value", dummy.id.version);
+		skip = run_embscript(p, elem, &dummy, L, swcfg->embscript);
+		if (skip < 0)
+			return -1;
+		if (skip)
+			continue;
+
+		/*
+		 * Store the variable in dictionary
+		 */
+		dict_set_value(&swcfg->vars, dummy.id.name, dummy.id.version);
+
+		TRACE("SWUpdate var: %s = %s",
+		       dummy.id.name,
+		       dict_get_value(&swcfg->vars, dummy.id.name));
+	}
+
+	return 0;
+}
+
+static int parse_vars(parsertype p, void *cfg, struct swupdate_cfg *swcfg, lua_State *L)
+{
+	void *setting;
+	const char *nodes[MAX_PARSED_NODES];
+
+	setting = find_node_and_path(p, cfg, "vars", swcfg, nodes);
+
+	if (!setting)
+		return 0;
+
+	return _parse_vars(p, cfg, setting, nodes, swcfg, L);
+}
+
 static int _parse_images(parsertype p, void *cfg, void *setting, const char **nodes, struct swupdate_cfg *swcfg, lua_State *L)
 {
 	void *elem;
@@ -941,7 +1033,8 @@ static int parser(parsertype p, void *cfg, struct swupdate_cfg *swcfg)
 		parse_files(p, cfg, swcfg, L) ||
 		parse_images(p, cfg, swcfg, L) ||
 		parse_scripts(p, cfg, swcfg, L) ||
-		parse_bootloader(p, cfg, swcfg, L);
+		parse_bootloader(p, cfg, swcfg, L) ||
+		parse_vars(p, cfg, swcfg, L);
 
 	/*
 	 * Move the partitions at the beginning to be processed
@@ -961,7 +1054,6 @@ static int parser(parsertype p, void *cfg, struct swupdate_cfg *swcfg)
 
 	return ret;
 }
-#endif
 
 #ifdef CONFIG_LIBCONFIG
 int parse_cfg (struct swupdate_cfg *swcfg, const char *filename)
@@ -976,11 +1068,7 @@ int parse_cfg (struct swupdate_cfg *swcfg, const char *filename)
 	/* Read the file. If there is an error, report it and exit. */
 	DEBUG("Parsing config file %s", filename);
 	if(config_read_file(&cfg, filename) != CONFIG_TRUE) {
-		printf("%s ", config_error_file(&cfg));
-		printf("%d ", config_error_line(&cfg));
-		printf("%s ", config_error_text(&cfg));
-
-		fprintf(stderr, "%s:%d - %s\n", config_error_file(&cfg),
+		ERROR("%s:%d - %s\n", config_error_file(&cfg),
 			config_error_line(&cfg), config_error_text(&cfg));
 		config_destroy(&cfg);
 		return -1;
@@ -1003,8 +1091,6 @@ int parse_cfg (struct swupdate_cfg __attribute__ ((__unused__)) *swcfg,
 }
 #endif
 
-#ifdef CONFIG_JSON
-
 #define JSON_OBJECT_FREED 1
 
 int parse_json(struct swupdate_cfg *swcfg, const char *filename)
@@ -1068,10 +1154,3 @@ int parse_json(struct swupdate_cfg *swcfg, const char *filename)
 
 	return ret;
 }
-#else
-int parse_json(struct swupdate_cfg __attribute__ ((__unused__)) *swcfg,
-		const char __attribute__ ((__unused__)) *filename)
-{
-	return -1;
-}
-#endif
