@@ -26,7 +26,7 @@
 #include <swupdate_status.h>
 #include "suricatta/suricatta.h"
 #include "suricatta/server.h"
-#include "suricatta_private.h"
+#include "server_utils.h"
 #include "parselib.h"
 #include "channel.h"
 #include <curl/curl.h>
@@ -594,7 +594,7 @@ static int server_general_settings(void *elem, void  __attribute__ ((__unused__)
 	get_field(LIBCFG_PARSER, elem, "polldelay",
 		&server_general.polling_interval);
 
-	suricatta_channel_settings(elem, &channel_data_defaults);
+	channel_settings(elem, &channel_data_defaults);
 
 	return 0;
 }
