@@ -22,7 +22,7 @@
 #include <pthread.h>
 #include "suricatta/suricatta.h"
 #include "suricatta/server.h"
-#include "suricatta_private.h"
+#include "server_utils.h"
 #include "parselib.h"
 #include "channel.h"
 #include "channel_curl.h"
@@ -31,10 +31,12 @@
 #include "parselib.h"
 #include "swupdate_settings.h"
 #include "swupdate_dict.h"
+#include "swupdate_vars.h"
 
 #define INITIAL_STATUS_REPORT_WAIT_DELAY 10
 
 #define JSON_OBJECT_FREED 1
+#define SERVER_NAME "hawkbit"
 
 static struct option long_options[] = {
     {"tenant", required_argument, NULL, 't'},
@@ -853,6 +855,30 @@ static server_op_res_t handle_feedback(int action_id, server_op_res_t result,
 	return SERVER_UPDATE_AVAILABLE;
 }
 
+static void get_action_id_from_env(int *action_id)
+{
+	/*
+	 * Retrieve the action_id if SWUpdate stored it
+	 * during the update
+	 * The current action_id on the server can be different if the operator
+	 * cancelled a rollout and started a new one while SWUpdate was
+	 * restarting.
+	 * Get the acction_id that corresponds to the done update if it was
+	 * stored.
+	 */
+	char *action_str = swupdate_vars_get("action_id", NULL);
+	if (action_str) {
+		int tmp = ustrtoull(action_str, NULL, 10);
+		/*
+		 * action_id = 0 is invalid, then check it
+		 */
+		if (tmp > 0) {
+			*action_id = tmp;
+			TRACE("Retrieve action_id from previous run: %d", *action_id);
+		}
+		free(action_str);
+	}
+}
 
 server_op_res_t server_handle_initial_state(update_state_t stateovrrd)
 {
@@ -899,11 +925,17 @@ server_op_res_t server_handle_initial_state(update_state_t stateovrrd)
 	server_op_res_t result;
 
 	/*
-	 * Retrieving current action id
+	 * Try to retrieve current action id
 	 */
+
 	channel_data_t channel_data = channel_data_defaults;
-	result =
-	    server_get_deployment_info(server_hawkbit.channel, &channel_data, &action_id);
+	result = server_get_deployment_info(server_hawkbit.channel,
+						    &channel_data, &action_id);
+
+	/*
+	 * Get action_id from env, if any
+	 */
+	get_action_id_from_env(&action_id);
 
 	result = handle_feedback(action_id, result, state, reply_result,
 				 reply_execution, 1, &reply_message);
@@ -911,6 +943,11 @@ server_op_res_t server_handle_initial_state(update_state_t stateovrrd)
 	if (result != SERVER_UPDATE_AVAILABLE)
 		return result;
 
+	/*
+	 * Everything fine, reset action_id if any
+	 */
+	swupdate_vars_set("action_id", NULL, NULL);
+
 	/* NOTE (Re-)setting STATE_KEY=STATE_OK == '0' instead of deleting it
 	 *      as it may be required for the switchback/recovery U-Boot logics.
 	 */
@@ -1060,6 +1097,17 @@ server_op_res_t server_process_update_artifact(int action_id,
 	    json_object_array_length(json_data_artifact);
 	int json_data_artifact_installed = 0;
 	json_object *json_data_artifact_item = NULL;
+
+	char *action_id_str;
+	if (asprintf(&action_id_str, "%d", action_id) == ENOMEM_ASPRINTF) {
+		ERROR("OOM reached when saving action_id");
+		return SERVER_EERR;
+	}
+	if (swupdate_vars_set("action_id", action_id_str, NULL)) {
+		WARN("Action_id cannot be stored, do yourself");
+	}
+	free(action_id_str);
+
 	for (int json_data_artifact_count = 0;
 	     json_data_artifact_count < json_data_artifact_max;
 	     json_data_artifact_count++) {
@@ -1147,6 +1195,7 @@ server_op_res_t server_process_update_artifact(int action_id,
 
 		static const char* const update_info = STRINGIFY(
 		{
+		"server": "%s",
 		"update": "%s",
 		"part": "%s",
 		"version": "%s",
@@ -1156,6 +1205,7 @@ server_op_res_t server_process_update_artifact(int action_id,
 		);
 		if (ENOMEM_ASPRINTF ==
 		    asprintf(&channel_data.info, update_info,
+			    SERVER_NAME,
 			    update_action,
 			    part,
 			    version,
@@ -1553,6 +1603,7 @@ static server_op_res_t server_send_target_data(void)
 	{
 		"id": "%s",
 		"time": "%s",
+		"mode" : "replace",
 		"status": {
 			"result": {
 				"finished": "%s"
@@ -1674,7 +1725,7 @@ static int server_hawkbit_settings(void *elem, void  __attribute__ ((__unused__)
 	get_field(LIBCFG_PARSER, elem, "initial-report-resend-period",
 		&server_hawkbit.initial_report_resend_period);
 
-	suricatta_channel_settings(elem, &channel_data_defaults);
+	channel_settings(elem, &channel_data_defaults);
 
 	get_field(LIBCFG_PARSER, elem, "usetokentodwl",
 		&server_hawkbit.usetokentodwl);
@@ -1951,12 +2002,18 @@ static server_op_res_t server_activation_ipc(ipc_message *msg)
 
 	json_object *json_data = json_get_path_key(
 	    json_root, (const char *[]){"id", NULL});
-	if (json_data == NULL) {
-		ERROR("Got malformed JSON: Could not find action id");
-		DEBUG("Got JSON: %s", json_object_to_json_string(json_data));
+
+	int action_id = -1;
+	if (json_data) {
+		action_id = json_object_get_int(json_data);
+	} else {
+		get_action_id_from_env(&action_id);
+	}
+
+	if (action_id < 0) {
+		ERROR("No action_id passed into JSON message and no action:_id in env");
 		return SERVER_EERR;
 	}
-	int action_id = json_object_get_int(json_data);
 
 	json_data = json_get_path_key(
 	    json_root, (const char *[]){"status", NULL});
@@ -2160,5 +2217,5 @@ server_t server_hawkbit_funcs = {
 __attribute__((constructor))
 static void register_server_hawkbit(void)
 {
-	register_server("hawkbit", &server_hawkbit_funcs);
+	register_server(SERVER_NAME, &server_hawkbit_funcs);
 }
