@@ -18,22 +18,19 @@
 #include <lualib.h>
 #include <lua_util.h>
 
-#ifdef CONFIG_JSON
 #include <json-c/json_visit.h>
-#endif
 
 #include <util.h>
 #include <network_ipc.h>
 #include <progress_ipc.h>
 #include <channel.h>
 #include <channel_curl.h>
-#include <swupdate.h>
 #include <parselib.h>
 #include <state.h>
 #include <bootloader.h>
 #include <swupdate_settings.h>
 #include <swupdate_dict.h>
-#include "suricatta_private.h"
+#include "server_utils.h"
 #include "suricatta/server.h"
 
 #define CONFIG_SECTION "suricatta"
@@ -63,11 +60,7 @@ static channel_data_t channel_data_defaults = {
 	.retry_sleep = CHANNEL_DEFAULT_RESUME_DELAY,
 	.retries = CHANNEL_DEFAULT_RESUME_TRIES,
 	.low_speed_timeout = 300,
-#ifdef CONFIG_JSON
-	.format = CHANNEL_PARSE_JSON,
-#else
-	.format = CHANNEL_PARSE_RAW,
-#endif
+	.format = CHANNEL_PARSE_JSON, /* just default, it can be overwritten */
 	.debug = false,
 #ifdef CONFIG_SURICATTA_SSL
 	.usessl = true,
@@ -274,8 +267,6 @@ typedef struct {
 					     get_from_table_3(__VA_ARGS__) \
 					     )
 
-
-#ifdef CONFIG_JSON
 /**
  * @brief Push the Lua equivalent of a JSON type to Table on stack top.
  *
@@ -379,8 +370,6 @@ static bool json_to_table(lua_State *L, json_object *json_root)
 	}
 	return true;
 }
-#endif
-
 
 /**
  * @brief Push true or nil to Lua stack according to result argument.
@@ -537,7 +526,11 @@ static void channel_push_options(lua_State *L, channel_data_t *channel_data)
 	push_to_table(L, "sslkey",             channel_data->sslkey);
 	push_to_table(L, "sslcert",            channel_data->sslcert);
 	push_to_table(L, "ciphers",            channel_data->ciphers);
-	push_to_table(L, "proxy",              channel_data->proxy);
+	if (channel_data->proxy && channel_data->proxy == USE_PROXY_ENV) {
+		push_to_table(L, "proxy",      "");
+	} else {
+		push_to_table(L, "proxy",      channel_data->proxy);
+	}
 	push_to_table(L, "info",               channel_data->info);
 	push_to_table(L, "auth_token",         channel_data->auth_token);
 	push_to_table(L, "content_type",       channel_data->content_type);
@@ -579,7 +572,6 @@ static void channel_set_options(lua_State *L, channel_data_t *channel_data)
 	get_from_table(L, "sslkey",             channel_data->sslkey, COPY_DEST);
 	get_from_table(L, "sslcert",            channel_data->sslcert, COPY_DEST);
 	get_from_table(L, "ciphers",            channel_data->ciphers, COPY_DEST);
-	get_from_table(L, "proxy",              channel_data->proxy, COPY_DEST);
 	get_from_table(L, "info",               channel_data->info, COPY_DEST);
 	get_from_table(L, "auth_token",         channel_data->auth_token, COPY_DEST);
 	get_from_table(L, "content_type",       channel_data->content_type, COPY_DEST);
@@ -600,6 +592,14 @@ static void channel_set_options(lua_State *L, channel_data_t *channel_data)
 		channel_data->max_download_speed = (unsigned int)ustrtoull(max_download_speed, NULL, 10);
 		free(max_download_speed);
 	}
+	lua_getfield(L, -1, "proxy");
+	if (lua_isstring(L, -1)) {
+		channel_data->proxy = strnlen(lua_tostring(L, -1), 1) > 0
+			? strdup(lua_tostring(L, -1)) : USE_PROXY_ENV;
+	} else {
+		channel_data->proxy = NULL;
+	}
+	lua_pop(L, 1);
 }
 
 
@@ -624,7 +624,9 @@ static void channel_free_options(channel_data_t *channel_data)
 	free(channel_data->sslkey);
 	free(channel_data->sslcert);
 	free(channel_data->ciphers);
-	free(channel_data->proxy);
+	if (channel_data->proxy && channel_data->proxy != USE_PROXY_ENV) {
+		free(channel_data->proxy);
+	}
 	free(channel_data->info);
 }
 
@@ -721,7 +723,6 @@ static int channel_do_operation(lua_State *L, channel_method_t op)
 	lua_newtable(L);
 	push_to_table(L, "http_response_code", channel_data.http_response_code);
 	push_to_table(L, "format",             channel_data.format);
-	#ifdef CONFIG_JSON
 	if (channel_data.format == CHANNEL_PARSE_JSON) {
 		lua_pushstring(L, "json_reply");
 		if (!channel_data.json_reply ||
@@ -735,7 +736,6 @@ static int channel_do_operation(lua_State *L, channel_method_t op)
 			ERROR("JSON object should be freed but was not.");
 		}
 	}
-	#endif
 	if (channel_data.format == CHANNEL_PARSE_RAW) {
 		lua_pushstring(L, "raw_reply");
 		if (!channel_data.raw_reply) {
@@ -904,7 +904,7 @@ static void *progress_offloader_thread(void *data)
 
 			(void)pthread_mutex_lock(thread_data->lua_lock);
 			lua_newtable(thread_data->L);
-			push_to_table(thread_data->L, "magic",       message->magic);
+			push_to_table(thread_data->L, "apiversion",  message->apiversion);
 			push_to_table(thread_data->L, "status",      message->status);
 			push_to_table(thread_data->L, "dwl_percent", message->dwl_percent);
 			push_to_table(thread_data->L, "nsteps",      message->nsteps);
@@ -914,7 +914,6 @@ static void *progress_offloader_thread(void *data)
 			push_to_table(thread_data->L, "hnd_name",    (char*)message->hnd_name);
 			push_to_table(thread_data->L, "source",      message->source);
 			push_to_table(thread_data->L, "info",        (char*)message->info);
-			#ifdef CONFIG_JSON
 			if (message->infolen > 0) {
 				lua_pushstring(thread_data->L, "jsoninfo");
 				struct json_object *json_root = json_tokener_parse(
@@ -928,7 +927,6 @@ static void *progress_offloader_thread(void *data)
 				}
 				lua_settable(thread_data->L, -3);
 			}
-			#endif
 			STAILQ_REMOVE_HEAD(&thread_data->progress_msgq, entries);
 			free(qitem);
 			(void)pthread_mutex_unlock(thread_data->progress_msgq_lock);
@@ -1635,6 +1633,10 @@ static int suricatta_lua_module(lua_State *L)
 	channel_push_options(L, &channel_data_defaults);
 	lua_settable(L, -3);
 
+	lua_pushstring(L, "USE_PROXY_ENV");
+	lua_pushstring(L, "");
+	lua_settable(L, -3);
+
 	lua_pushstring(L, "content");
 	lua_newtable(L);
 	push_to_table(L, "RAW",  CHANNEL_PARSE_RAW);
@@ -1986,7 +1988,6 @@ static server_op_res_t server_ipc(ipc_message *msg)
 	lua_pushstring(gL, "msg");
 	lua_pushlstring(gL, (char *)msg->data.procmsg.buf, msg->data.procmsg.len);
 	lua_settable(gL, -3);
-	#ifdef CONFIG_JSON
 	if (msg->data.procmsg.len > 0) {
 		lua_pushstring(gL, "json");
 		struct json_object *json_root = json_tokener_parse(msg->data.procmsg.buf);
@@ -1999,7 +2000,6 @@ static server_op_res_t server_ipc(ipc_message *msg)
 			ERROR("JSON object should be freed but was not.");
 		}
 	}
-	#endif
 	server_op_res_t result = map_lua_result(call_lua_func(gL, SURICATTA_FUNC_IPC, 1));
 	msg->type = result == SERVER_OK ? ACK : NACK;
 	msg->data.procmsg.len = 0;
